﻿using System;
using Autofac;
using Xunit;
using PX.Data;
using PX.Data.Unit;
using PX.Objects.CM.Extensions;
using PX.Objects.Unit;

namespace TestSample
{
	public class TestExtension : TestBase
	{
		protected override void RegisterServices(ContainerBuilder builder)
		{
			base.RegisterServices(builder);
			builder
				.Register<Func<PXGraph, IPXCurrencyService>>(context =>
				{
					return (graph) => new CurrencyServiceMock();
				});
		}
		[Fact]
		public void DisabledForBase()
		{
			GraphMock graph = PXGraph.CreateInstance<GraphMock>();
			BindingMock extension = graph.GetExtension<BindingMock>();
			DocumentMock document = graph.Documents
				.Insert().GetExtension<DocumentMock>();
			CurrencyInfo info = extension.currencyinfo.Current;
			PXCache cache = extension.currencyinfo.Cache;
			Assert.False(((PXFieldState)cache.GetStateExt<CurrencyInfo.curyRateTypeID>(info)).Enabled);
			Assert.False(((PXFieldState)cache.GetStateExt<CurrencyInfo.curyEffDate>(info)).Enabled);
			Assert.False(((PXFieldState)cache.GetStateExt<CurrencyInfo.curyMultDiv>(info)).Enabled);
		}
		[Fact]
		public void EnabledForForeign()
		{
			GraphMock graph = PXGraph.CreateInstance<GraphMock>();
			BindingMock extension = graph.GetExtension<BindingMock>();
			DocumentMock document = graph.Documents
				.Insert().GetExtension<DocumentMock>();
			CurrencyInfo info = extension.currencyinfo.Current;
			info.CuryID = "EUR";
			extension.currencyinfo.Update(info);
			PXCache cache = extension.currencyinfo.Cache;
			Assert.True(((PXFieldState)cache.GetStateExt<CurrencyInfo.curyRateTypeID>(info)).Enabled);
			Assert.True(((PXFieldState)cache.GetStateExt<CurrencyInfo.curyEffDate>(info)).Enabled);
			Assert.True(((PXFieldState)cache.GetStateExt<CurrencyInfo.curyMultDiv>(info)).Enabled);
		}
		[Theory]
		[InlineData("EUR", "BANK", false, false)]
		[InlineData("JPY", "SPOT", true, true)]
		public void DefaultFromBAccount(string curyID, string curyRateTypeID, bool curyEnabled, bool rateEnabled)
		{
			GraphMock graph = PXGraph.CreateInstance<GraphMock>();
			BindingMock extension = graph.GetExtension<BindingMock>();
			BAccountMock source = graph.Source
				.Insert().GetExtension<BAccountMock>();
			source.CuryID = curyID;
			source.CuryRateTypeID = curyRateTypeID;
			source.AllowOverrideCury = curyEnabled;
			source.AllowOverrideRate = rateEnabled;
			graph.Source
				.Update(graph.Source.Current);
			DocumentMock doc = graph.Documents
				.Insert().GetExtension<DocumentMock>();
			CurrencyInfo info = extension.currencyinfo.Current;
			PXCache cache = extension.currencyinfo.Cache;
			Assert.Equal(info.CuryID, curyID);
			Assert.Equal(info.CuryRateTypeID, curyRateTypeID);
			Assert.Equal(((PXFieldState)graph.Documents.Cache.GetStateExt<DocumentMock.curyID>(graph.Documents.Current)).Enabled, curyEnabled);
			Assert.Equal(((PXFieldState)cache.GetStateExt<CurrencyInfo.sampleCuryRate>(info)).Enabled, rateEnabled);
		}
		[Theory]
		[InlineData(63.40, 81.15)]
		public void CalculateBaseAmount(decimal curyamt, decimal baseamt)
		{
			GraphMock graph = PXGraph.CreateInstance<GraphMock>();
			BindingMock extension = graph.GetExtension<BindingMock>();
			DocumentMock doc = graph.Documents
				.Insert().GetExtension<DocumentMock>();
			doc.CuryField = curyamt;
			graph.Documents
				.Update(graph.Documents.Current);
			DetailMock det1 = graph.Details
				.Insert().GetExtension<DetailMock>();
			det1.CuryField = curyamt;
			graph.Details
				.Update(graph.Details.Current);
			Assert.Equal(doc.BaseField, curyamt);
			Assert.Equal(det1.BaseField, curyamt);
			CurrencyInfo info = extension.currencyinfo.Current;
			info.CuryID = "EUR";
			extension.currencyinfo.Update(info);
			DetailMock det2 = graph.Details
				.Insert().GetExtension<DetailMock>();
			det2.CuryField = curyamt;
			graph.Details
				.Update(graph.Details.Current);
			graph.Persist();
			Assert.Equal(doc.BaseField, baseamt);
			Assert.Equal(det1.BaseField, baseamt);
			Assert.Equal(det2.BaseField, baseamt);
		}
		[Theory]
		[InlineData("EUR", 63.40, 81.15)]
		[InlineData("USD", 63.40, 63.40)]
		public void ViewBaseAmount(string curyID, decimal curyamt, decimal baseamt)
		{
			GraphMock graph = PXGraph.CreateInstance<GraphMock>();
			BindingMock extension = graph.GetExtension<BindingMock>();
			var doc = graph.Documents
				.Insert().GetExtension<DocumentMock>();
			doc.CuryField = curyamt;
			graph.Documents
				.Update(graph.Documents.Current);
			var det = graph.Details
				.Insert().GetExtension<DetailMock>();
			det.CuryField = curyamt;
			graph.Details
				.Update(graph.Details.Current);
			CurrencyInfo info = extension.currencyinfo.Current;
			info.CuryID = curyID;
			extension.currencyinfo.Update(info);
			extension.currencyView.Press();
			Assert.Equal(graph.Documents.GetValueExt<DocumentMock.curyField>(graph.Documents.Current), baseamt);
			Assert.Equal(graph.Details.GetValueExt<DetailMock.curyField>(graph.Details.Current), baseamt);
		}
	}
}
